#!/bin/sh
############################################################################
##
##  Filename:
##            TestIt.sh
##
##  Authors: 
##            Sven O. Moch and Peter Uwer
##
##  Purpose: 
##            Simple shell script to run the examples
##
##  Comments:
##
##
##  Revision: 
##            $Id: TestIt.sh,v 1.0 2005/08/02 10:09:54 puwer Exp $
##
##            $Log: TestIt.sh,v $
##            Revision 1.0  2005/08/02 10:09:54  puwer
##            Initial revision
##
##
############################################################################


RM_BIN="/bin/rm"
RM_OPTIONS="-i"
FORM_BIN=`which form`

if [ ! -x ${RM_BIN} ]; then
  echo  >&2 "Cannot find binary for rm, please set RM_BIN"
  exit 1
fi

if [ ! -x ${FORM_BIN} ]; then
  echo >&2 "Cannot find form binary, please set FORM_BIN"
  exit 1
fi


FORMPATH=.:../src:../summer
export FORMPATH

tmpfiles="Examples.log Examples.out ctoponf343 formoutputctopo.log bbox86 formoutputbbox.log Trimass18 formoutputtrimass.log"
cd tmp
echo "Remove temporary files..."
${RM_BIN} ${RM_OPTIONS} $tmpfiles
cd ..


cat << END
-------------------------------------------------------------
We will now run a few form scripts to make sure that
everything works fine. Depending on the performance
of your machine this may take some time.
-------------------------------------------------------------
END

echo "Expand generalized hypergeomtric functions..."

time ${FORM_BIN} Examples.frm > ./tmp/Examples.log

echo "Calculate a few loop integrals appearing in 3-Jet@NNLO..."

echo "Calculate the C-topology"
time ${FORM_BIN} -d TYPE=CTOPO -d INTEGRAL=343 DoIntegrals.frm > ./tmp/formoutputctopo.log

echo "Calculate the B-box"
time ${FORM_BIN} -d TYPE=BBOX -d INTEGRAL=86 DoIntegrals.frm > ./tmp/formoutputbbox.log

echo "Calculate a triangle"
time ${FORM_BIN} -d TYPE=TRIMASS -d INTEGRAL=18 DoIntegrals.frm > ./tmp/formoutputtrimass.log

# Check here against reference result
echo "Compare with know results..."
time ${FORM_BIN} CheckResults.frm


#########################################################################
# Local variables:
# modify-insert-tag: nil
# comment-start: "#"
# tab-width: 2
# End:
#########################################################################
