#procedure AlgD(i)
****************************************************************************
**
**  Filename:
**            AlgD.prc
**
**  Authors: 
**            Sven O. Moch and Peter Uwer
**
**  Purpose:
**            Implementation of the binomial convolution algorithm (type D) 
**            to perform sums of the form
**            \begin{eqnarray*}
**                &&\sum\limits_{i=1}^{n-1}\, 
**                \left(\begin{array}{c} n \\ i \end{array} \right)\, 
**                 (-1)^i {x_1^i \over i^{m_1}} 
**                  S(i;m_2,...,m_k;x_2,...,x_k)\\
**                &&\hspace{1cm}\times 
**                {(x^\prime_1)^{n-i} \over (n-i)^{m^{\prime}_1}} 
**                 S(n-i;m^{\prime}_2,...,m^{\prime}_l;
**                      x^{\prime}_2,...,x^{\prime}_l)\, 
**            \end{eqnarray*}
**            for any (integer) values of $m_i,m_i^\prime \ge 0$ and $m_1>0$.
**            The argument specifies 
**            the index $i$ of the sum to be done. 
**            The special case $m_1=0$ is treated in \keyword{AlgD0}.
**
**  Comments:
**
**
**  Revision: 
**            $Id: AlgD.prc,v 1.0 2005/08/02 10:09:54 puwer Exp $
**
**            $Log: AlgD.prc,v $
**            Revision 1.0  2005/08/02 10:09:54  puwer
**            Initial revision
**
**
****************************************************************************
*
if ( count(den`i',1) == 1 );
	if ( count(S`i',1) == 2 ); 
		if ( count(pow`i',1) == 1 );
			id  sum`i'(j`i',1,n1?)*S`i'(R(?a),X(?x),n2?,-j`i')*S`i'(R(?b),X(?y),n0?,j`i')*
				bino`i'(n3?,0,j`i')*den`i'(k?,n0?,j`i')*pow`i'(y1?,j`i') =
				SAlgDS(R(?a),X(?x),R(?b),X(?y),y1,k,n0,n1,n2,n3);
			id  sum`i'(j`i',0,n1?)*S`i'(R(?a),X(?x),n2?,-j`i')*S`i'(R(?b),X(?y),n0?,j`i')*
				bino`i'(n3?,0,j`i')*den`i'(k?{,0},n0?,j`i')*pow`i'(y1?,j`i') =
				SAlgDS(R(?a),X(?x),R(?b),X(?y),y1,k,n0,n1,n2,n3);
		else;
			id  sum`i'(j`i',1,n1?)*S`i'(R(?a),X(?x),n2?,-j`i')*S`i'(R(?b),X(?y),n0?,j`i')*
				bino`i'(n3?,0,j`i')*den`i'(k?,n0?,j`i') =
				SAlgDS(R(?a),X(?x),R(?b),X(?y),1,k,n0,n1,n2,n3);
			id  sum`i'(j`i',0,n1?)*S`i'(R(?a),X(?x),n2?,-j`i')*S`i'(R(?b),X(?y),n0?,j`i')*
				bino`i'(n3?,0,j`i')*den`i'(k?{,0},n0?,j`i') =
				SAlgDS(R(?a),X(?x),R(?b),X(?y),1,k,n0,n1,n2,n3);
		endif;
	endif;
	id  SAlgDS(?a,n2?{,0},n0?,n0?,n0?) = SAlgDS(?a,n2,n0);
	id  SAlgDS(?a,n2?{,0},n1?,n0?,n0?) = SAlgDS(?a,n2,n1+1,n0,n0);
	id  SAlgDS(?a,n2?{,0},n0?,n0?,n0?) = SAlgDS(?a,n2,n0);
	id  SAlgDS(?a,n2?{,0},n1?,n0?,n0?) = SAlgDS(?a,n2,n1-1,n0,n0);
	id  SAlgDS(R(?a),X(?x),R(?b),X(?y),1,k?,n0?,n1?,n2?,n3?) = sum`i'(j`i',1,n1)*
		S`i'(R(?a),X(?x),n2,-j`i')*S`i'(R(?b),X(?y),n0,j`i')*
		bino`i'(n3,j`i')*den`i'(n0,j`i')^k;
	id  SAlgDS(R(?a),X(?x),R(?b),X(?y),y1?,k?,n0?,n1?,n2?,n3?) = sum`i'(j`i',1,n1)*
		S`i'(R(?a),X(?x),n2,-j`i')*S`i'(R(?b),X(?y),n0,j`i')*
		bino`i'(n3,j`i')*den`i'(n0,j`i')^k*pow`i'(y1,j`i');
endif;

*
*   - sums this routine can handle are now in the function SAlgDS:
*       * add pairs of x1,k,n if there are new denominators and 
*         summations boundaries.
*       * eventually hit on sums that can be done and then work 
*         backwards 
*

id  SAlgDS(R(?a),X(?x),R(?b),X(?y),x1?,k?,n0?,N?) = SAlgDS(RR,XX,R(?a),X(?x),R(?b),X(?y),x1,k,n0,N);

*
* --> now comes the great recursion
*
repeat;
*
* get rid of powers of j`i' in denominator
*
	repeat id  SAlgDS(RR(?c),XX(?z),R(n1?pos_,?a),X(x1?,?x),R(?b),X(?y),y1?,k?pos_,n0?,N?) =
		+ SAlgDS(RR(?c,1),XX(?z,1),R(n1,?a),X(x1,?x),R(?b),X(?y),y1,k-1,n0,N)
		+ delta_(n1-1)*
			SAlgDS(RR(?c,1),XX(?z,x1),R(?a),X(?x),R(?b),X(?y),y1/x1,k,n0,N)
		+ theta_(n1-2)*(
			sum_(jjj,1,k,
				SAlgDS(RR(?c,k+n1-jjj),XX(?z,x1),R(?a),X(?x),R(?b),X(?y),y1/x1,jjj,n0,N)*
				binom_(k+n1-2-jjj,n1-2))
		+ sum_(jjj,1,n1-1,
				SAlgDS(RR(?c,k+n1-jjj),XX(?z,y1),R(?b),X(?y),R(?a),X(?x),x1/y1,jjj,n0,N)*
				binom_(k+n1-2-jjj,k-1)));
*
* take out cases where either  S`i'(..,0,j`i') or  S`i'(..,N-j`i') has vanished
*
	id SAlgDS(RR(?c),XX(?z),R,X,R(?b),X(?y),y1?,k?,n0?,N?) = 
		+ SAlgDtoCS(RR(?c),XX(?z),R(?b),X(?y),y1,k,n0,N);
	id SAlgDS(RR,XX,R(?b),X(?y),R,X,y1?,0,n0?,N?) = 
		+ pow(y1,N)*SAlgDtoCS(RR,XX,R(?b),X(?y),1/y1,0,n0,N);
	id SAlgDS(RR(?c,n1?),XX(?z,x1?),R(?b),X(?y),R,X,y1?,0,n0?,N?) = 
		+ SAlgDtoCS(RR(?c,n1),XX(?z,x1*y1),R(?b),X(?y),1/y1,0,n0,N);

*
* go back to first pattern with reduced weight
*
	id,once,  SAlgDS(RR,XX,R(n1?,?a),X(x1?,?x),R(m1?,?b),X(y1?,?y),-1,0,0,N?) = 
		+ SAlgDS(RR,XX,R(n1,?a),X(x1,?x),R(?b),X(?y),-y1,m1-1,0,N)*den(N) 
		+ SAlgDS(RR,XX,R(m1,?b),X(y1,?y),R(?a),X(?x),-x1,n1-1,0,N)*den(N)*pow(-1,N);
	id,once,  SAlgDS(RR(?c,n2?),XX(?z,x2?),R(n1?,?a),X(x1?,?x),R(m1?,?b),X(y1?,?y),-1,0,0,N?) = 
		+ SAlgDS(RR(?c,n2+1),XX(?z, x2),R(n1,?a),X(x1,?x),R(?b),X(?y),-y1,m1-1,0,N)
		+ SAlgDS(RR(?c,n2+1),XX(?z,-x2),R(m1,?b),X(y1,?y),R(?a),X(?x),-x1,n1-1,0,N);

	id,once,  SAlgDS(RR,XX,R(n1?,?a),X(x1?,?x),R(m1?,?b),X(y1?,?y),x0?!{,-1},0,0,N?) =
		+ SAlgDS(RR(1),XX(  den(1+x0)),R(n1,?a),X(x1,?x),R(?b),X(?y),y1*x0,m1-1,0,N)*pow(1+x0,N)
		+ SAlgDS(RR(1),XX(1-den(1+x0)),R(m1,?b),X(y1,?y),R(?a),X(?x),x1/x0,n1-1,0,N)*pow(1+x0,N);
	id,once,  SAlgDS(RR(?c,m2?),XX(?z,y2?),R(n1?,?a),X(x1?,?x),R(m1?,?b),X(y1?,?y),x0?!{,-1},0,0,N?) =
		+ SAlgDS(RR(?c,m2,1),XX(?z,y2*num(1+x0),den(1+x0)),
			R(n1,?a),X(x1,?x),R(?b),X(?y),y1*x0,m1-1,0,N)
		+ SAlgDS(RR(?c,m2,1),XX(?z,y2*num(1+x0),1-den(1+x0)),
			R(m1,?b),X(y1,?y),R(?a),X(?x),x1/x0,n1-1,0,N);

endrepeat;

*
* resolve SAlgDtoCS with AlgC
*
id SAlgDtoCS(RR,XX,R(?b),X(?y),x1?,k?,n0?{,0},N?) = 
	+ sum`i'(j`i',1,N) * bino`i'(N,0,j`i') * pow`i'(x1,j`i') * 
	den`i'(k,0,j`i') * S`i'(R(?b),X(?y),0,j`i')
	- pow(x1,N) * den(N)^k * S(R(?b),X(?y),N);

id SAlgDtoCS(RR(?a,m1?),XX(?x,y1?),R(?b),X(?y),x1?,k?,n0?{,0},N?) = 
	+ SAlgDtoCS(RR(?a,m1),XX(?x,y1),N,Nx)*
	sum`i'(j`i',1,Nx) * bino`i'(Nx,0,j`i') * pow`i'(x1,j`i') * 
	den`i'(k,0,j`i') * S`i'(R(?b),X(?y),0,j`i')
	- S(R(?a,m1+k,?b),X(?x,y1*x1,?y),N);

#call AlgC(`i')

*
* do final summation over result of AlgC
*
repeat id pow(x1?,Nx)*pow(x2?,Nx) = pow(x1*x2,Nx);
repeat id den(Nx)*SAlgDtoCS(RR(?a,n1?),XX(?x,x1?),N?,Nx) = SAlgDtoCS(RR(?a,n1+1),XX(?x,x1),N,Nx);
id pow(x2?,Nx)*SAlgDtoCS(RR(?a,n1?),XX(?x,x1?),N?,Nx) = SAlgDtoCS(RR(?a,n1),XX(?x,x1*x2),N,Nx);
id SAlgDtoCS(RR(?a),XX(?x),N?,Nx)*S(R(?b),X(?y),Nx) = S(R(?a,?b),X(?x,?y),N);

#endprocedure
*************************************************************************
* Local variables:
* modify-insert-tag: nil
* comment-start: "*"
* tab-width: 2
* End:
*************************************************************************
